/*
 * JavaScript for lsystem.html
 * Copyright 2008, John Snyders
 * Creative Commons Attribution 3.0 United States License
 */

(function() {

var canvas = null; // the canvas
var t = null; // the turtle
var rendering = null; // timer used for generating and rendering

var colorMap = {
  'aliceblue': '#f0f8ff',
  'antiquewhite': '#faebd7',
  'aqua': '#00ffff',
  'aquamarine': '#7fffd4',
  'azure': '#f0ffff',
  'beige': '#f5f5dc',
  'bisque': '#ffe4c4',
  'black': '#000000',
  'blanchedalmond': '#ffebcd',
  'blue': '#0000ff',
  'blueviolet': '#8a2be2',
  'brown': '#a52a2a',
  'burlywood': '#deb887',
  'cadetblue': '#5f9ea0',
  'chartreuse': '#7fff00',
  'chocolate': '#d2691e',
  'coral': '#ff7f50',
  'cornflowerblue': '#6495ed',
  'cornsilk': '#fff8dc',
  'crimson': '#dc143c',
  'cyan': '#00ffff',
  'darkblue': '#00008b',
  'darkcyan': '#008b8b',
  'darkgoldenrod': '#b8860b',
  'darkgray': '#a9a9a9',
  'darkgreen': '#006400',
  'darkgrey': '#a9a9a9',
  'darkkhaki': '#bdb76b',
  'darkmagenta': '#8b008b',
  'darkolivegreen': '#556b2f',
  'darkorange': '#ff8c00',
  'darkorchid': '#9932cc',
  'darkred': '#8b0000',
  'darksalmon': '#e9967a',
  'darkseagreen': '#8fbc8f',
  'darkslateblue': '#483d8b',
  'darkslategray': '#2f4f4f',
  'darkslategrey': '#2f4f4f',
  'darkturquoise': '#00ced1',
  'darkviolet': '#9400d3',
  'deeppink': '#ff1493',
  'deepskyblue': '#00bfff',
  'dimgray': '#696969',
  'dimgrey': '#696969',
  'dodgerblue': '#1e90ff',
  'firebrick': '#b22222',
  'floralwhite': '#fffaf0',
  'forestgreen': '#228b22',
  'fuchsia': '#ff00ff',
  'gainsboro': '#dcdcdc',
  'ghostwhite': '#f8f8ff',
  'gold': '#ffd700',
  'goldenrod': '#daa520',
  'gray': '#808080',
  'green': '#008000',
  'greenyellow': '#adff2f',
  'grey': '#808080',
  'honeydew': '#f0fff0',
  'hotpink': '#ff69b4',
  'indianred': '#cd5c5c',
  'indigo': '#4b0082',
  'ivory': '#fffff0',
  'khaki': '#f0e68c',
  'lavender': '#e6e6fa',
  'lavenderblush': '#fff0f5',
  'lawngreen': '#7cfc00',
  'lemonchiffon': '#fffacd',
  'lightblue': '#add8e6',
  'lightcoral': '#f08080',
  'lightcyan': '#e0ffff',
  'lightgoldenrodyellow': '#fafad2',
  'lightgray': '#d3d3d3',
  'lightgreen': '#90ee90',
  'lightgrey': '#d3d3d3',
  'lightpink': '#ffb6c1',
  'lightsalmon': '#ffa07a',
  'lightseagreen': '#20b2aa',
  'lightskyblue': '#87cefa',
  'lightslategray': '#778899',
  'lightslategrey': '#778899',
  'lightsteelblue': '#b0c4de',
  'lightyellow': '#ffffe0',
  'lime': '#00ff00',
  'limegreen': '#32cd32',
  'linen': '#faf0e6',
  'magenta': '#ff00ff',
  'maroon': '#800000',
  'mediumaquamarine': '#66cdaa',
  'mediumblue': '#0000cd',
  'mediumorchid': '#ba55d3',
  'mediumpurple': '#9370db',
  'mediumseagreen': '#3cb371',
  'mediumslateblue': '#7b68ee',
  'mediumspringgreen': '#00fa9a',
  'mediumturquoise': '#48d1cc',
  'mediumvioletred': '#c71585',
  'midnightblue': '#191970',
  'mintcream': '#f5fffa',
  'mistyrose': '#ffe4e1',
  'moccasin': '#ffe4b5',
  'navajowhite': '#ffdead',
  'navy': '#000080',
  'oldlace': '#fdf5e6',
  'olive': '#808000',
  'olivedrab': '#6b8e23',
  'orange': '#ffa500',
  'orangered': '#ff4500',
  'orchid': '#da70d6',
  'palegoldenrod': '#eee8aa',
  'palegreen': '#98fb98',
  'paleturquoise': '#afeeee',
  'palevioletred': '#db7093',
  'papayawhip': '#ffefd5',
  'peachpuff': '#ffdab9',
  'peru': '#cd853f',
  'pink': '#ffc0cb',
  'plum': '#dda0dd',
  'powderblue': '#b0e0e6',
  'purple': '#800080',
  'red': '#ff0000',
  'rosybrown': '#bc8f8f',
  'royalblue': '#4169e1',
  'saddlebrown': '#8b4513',
  'salmon': '#fa8072',
  'sandybrown': '#f4a460',
  'seagreen': '#2e8b57',
  'seashell': '#fff5ee',
  'sienna': '#a0522d',
  'silver': '#c0c0c0',
  'skyblue': '#87ceeb',
  'slateblue': '#6a5acd',
  'slategray': '#708090',
  'slategrey': '#708090',
  'snow': '#fffafa',
  'springgreen': '#00ff7f',
  'steelblue': '#4682b4',
  'tan': '#d2b48c',
  'teal': '#008080',
  'thistle': '#d8bfd8',
  'tomato': '#ff6347',
  'turquoise': '#40e0d0',
  'violet': '#ee82ee',
  'wheat': '#f5deb3',
  'white': '#ffffff',
  'whitesmoke': '#f5f5f5',
  'yellow': '#ffff00',
  'yellowgreen': '#9acd32'
};


//
// basic progress control
//

function progressDraw(ctrl)
{
  var w = $(ctrl).width();
  var data = ctrl.progressCtl;
  var offset = Math.round(data.cur * (w / data.max));
  $(ctrl).find("div").css({"margin-left": (-w + offset) + "px"});
}

function progressInit()
{
  $("#genProgress, #renderProgress").html('<div><\/div>').each(function() {
    this.progressCtl = {cur: 0, max: 1};
    progressDraw(this);
  });
  $(window).resize(function(){
    // redraw control on resize
    $("#genProgress, #renderProgress").each(function() {
      progressDraw(this);
    });
  });
  $("#progressArea").hide();
}

function progressHide()
{
  $("#progressArea").fadeOut('slow');
}

function progressShow()
{
  $("#progressArea").fadeIn('slow');
}

function progressStart(name, max)
{
  $(name).each(function() {
    var data = this.progressCtl;
    var w = $(this).width();
    data.max = max;
    data.cur = 0;
    progressDraw(this);
  });
}

function progressSetMax(name, max)
{
  $(name).each(function() {
    var data = this.progressCtl;
    if (data.cur !== 0)
    {
      data.cur = max / data.max * data.cur;
    }
    data.max = max;
    progressDraw(this);
  });
}

function progressStep(name)
{
  $(name).each(function() {
    var data = this.progressCtl;
    var w = $(this).width();
    var offset;
    data.cur++;
    progressDraw(this);
  });
}
//
// end progress control
//

function clearInfo()
{
  $("#info").html("<p>Nothing to display. Choose a L-system model, adjust settings, and press play.<\/p>");
}

function clearCanvas()
{
  var bg = $("#backgroundColor").val();
  t.clear(bg);
  lastBg = bg;
}

function updateInfo(ls)
{
  $("#info").html("<p>Current Generation: " + ls.generation + ".<\/p>" +
      "<p>Number of modules: " + ls.moduleCount + ".<\/p>" +
      "<p>Generate time: " + ls.generateTime/1000 + " sec.<\/p>" +
      "<p>Render time: " + ls.renderTime/1000 + " sec.<\/p>" +
      "<p>Elapsed time: " + ls.elapsedTime/1000 + " sec.<\/p>" +
      "<p>Generation data:<p>" +
      "<textarea readonly='readonly' rows='8' cols='50' id='generationInfo'>" + 
      ls.curgenToString(", ") + "<\/textarea>");
}

//
// validation
//
function required(val)
{
  val = $.trim(val);
  return val.length > 0;
}

var integerRE = new RegExp("^[0-9]*$");

function validateInteger(val)
{
  val = $.trim(val);
  var i = parseInt(val, 10);
  return !isNaN(i) && integerRE.test(val);
}

function validateIntegerRange(val, min, max)
{
  val = $.trim(val);
  var i = parseInt(val, 10);
  return !isNaN(i) && i >= min && i <= max && integerRE.test(val);
}

var realRE = new RegExp("^[0-9,+,-,\\.,E,e]*$");

function validateReal(val)
{
  val = $.trim(val);
  var r = parseFloat(val);
  return !isNaN(r) && realRE.test(val);
}

function validateRealRange(val, min, max)
{
  val = $.trim(val);
  var r = parseFloat(val);
  return !isNaN(r) && r >= min && r <= max && realRE.test(val);
}

var colorRE = new RegExp("^#[0-9,a-f,A-F]{6}$|^#[0-9,a-f,A-F]{3}$");

function validateColor(val)
{
  val = $.trim(val);
  var valid = colorRE.test(val);
  if (!valid) {
    if (colorMap[val]) {
      valid = true;
    }
  }
  return valid;
}

var validations = {
    KochSnowflake_ngen: function(val) { return required(val) && validateInteger(val); },
    KochCurve_ngen: function(val) { return required(val) && validateInteger(val); },
    DragonCurve_d: function(val) { return required(val) && validateReal(val); },
    DragonCurve_ngen: function(val) { return required(val) && validateInteger(val); },
    SierpinskiTriangle_ngen: function(val) { return required(val) && validateInteger(val); },
    SierpinskiTriangle2_ngen: function(val) { return required(val) && validateInteger(val); },
    KochIsland_ngen: function(val) { return required(val) && validateInteger(val); },
    IslandsAndLakes_ngen: function(val) { return required(val) && validateInteger(val); },
    QuadSnowflake_ngen: function(val) { return required(val) && validateInteger(val); },
    abopFig1_9a_ngen: function(val) { return required(val) && validateInteger(val); },
    abopFig1_9b_ngen: function(val) { return required(val) && validateInteger(val); },
    abopFig1_9d_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree_a_dist: function(val) { return required(val) && validateReal(val); },
    Tree_a_angle: function(val) { return required(val) && validateReal(val); },
    Tree_a_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree_b_dist: function(val) { return required(val) && validateReal(val); },
    Tree_b_angle: function(val) { return required(val) && validateReal(val); },
    Tree_b_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree_c_dist: function(val) { return required(val) && validateReal(val); },
    Tree_c_angle: function(val) { return required(val) && validateReal(val); },
    Tree_c_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree_d_dist: function(val) { return required(val) && validateReal(val); },
    Tree_d_angle: function(val) { return required(val) && validateReal(val); },
    Tree_d_pl: function(val) { return required(val) && validateRealRange(val,0,1); },
    Tree_d_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree_e_dist: function(val) { return required(val) && validateReal(val); },
    Tree_e_angle: function(val) { return required(val) && validateReal(val); },
    Tree_e_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree_f_dist: function(val) { return required(val) && validateReal(val); },
    Tree_f_angle: function(val) { return required(val) && validateReal(val); },
    Tree_f_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree_1_27_dist: function(val) { return required(val) && validateReal(val); },
    Tree_1_27_angle: function(val) { return required(val) && validateReal(val); },
    Tree_1_27_ngen: function(val) { return required(val) && validateInteger(val); },
    RowOfTreesA_angle: function(val) { return required(val) && validateReal(val); },
    RowOfTreesA_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree1_31a_dist: function(val) { return required(val) && validateReal(val); },
    Tree1_31a_angle: function(val) { return required(val) && validateReal(val); },
    Tree1_31a_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree1_31b_dist: function(val) { return required(val) && validateReal(val); },
    Tree1_31b_angle: function(val) { return required(val) && validateReal(val); },
    Tree1_31b_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree1_31c_dist: function(val) { return required(val) && validateReal(val); },
    Tree1_31c_angle: function(val) { return required(val) && validateReal(val); },
    Tree1_31c_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree1_31d_dist: function(val) { return required(val) && validateReal(val); },
    Tree1_31d_angle: function(val) { return required(val) && validateReal(val); },
    Tree1_31d_ngen: function(val) { return required(val) && validateInteger(val); },
    Tree1_31e_dist: function(val) { return required(val) && validateReal(val); },
    Tree1_31e_angle: function(val) { return required(val) && validateReal(val); },
    Tree1_31e_ngen: function(val) { return required(val) && validateInteger(val); },
    penColor: function(val) { return required(val) && validateColor(val); },
    backgroundColor: function(val) { return required(val) && validateColor(val); },
    penWidth: function(val) { return required(val) && validateIntegerRange(val, 1, 10); }
  };

function validateField(ctrl)
{
    var val = $(ctrl).val();
    var id = $(ctrl).attr("id");
    if (!validations[id](val))
    {
      // TODO describe error
      $(ctrl).addClass("error");
      return false;
    }
    // else
    $(ctrl).removeClass("error");
    return true;
}

function allFieldsValid()
{
  var valid = true;
  var firstInvalidCtrl = null;
  var ctrl;
  for (id in validations) if (validations.hasOwnProperty(id))
  {
    ctrl = $("#" + id + ":enabled").get(0);
    if (ctrl && !validateField(ctrl))
    {
      valid = false;
      if (firstInvalidCtrl === null)
      {
        firstInvalidCtrl = ctrl;
      }
    }
  }
  if (!valid)
  {
    alert("One or more inputs are invalid.");
    $(firstInvalidCtrl).focus();
  }
  return valid;
}

//
// Access to control values
//
function getDelay()
{
  var delay = $("#delay").sliderInstance().interaction.curValue;
  return delay;
}

function getModulesPerStep()
{
  var mps = $("#modulesPerStep").sliderInstance().interaction.curValue;
  // the cur value is not offset by the min value.
  return mps + 1;
}

function getPenColor()
{
  var color = $("#penColor").val();
  return color;
}

function getPenWidth()
{
  var width = parseInt($("#penWidth").val(), 10);
  return width;
}

function getNumGen()
{
  var which = $("#whichLsys").val();
  var ctl = $("#" + which + "_ngen").get();
  var n = $(ctl).val();
  return n;
}

function decNextGen()
{
  var which = $("#whichLsys").val();
  var ctl = $("#" + which + "_ngen").get(0);
  var n = $(ctl).val();
  if (n > 0)
  {
    n--;
  }
  $(ctl).val(n);
  
  return n;
}

function incNextGen()
{
  var which = $("#whichLsys").val();
  var ctl = $("#" + which + "_ngen").get(0);
  var n = $(ctl).val();
  n++;
  $(ctl).val(n);

  return n;
}

//
// Actions
//
function save()
{
  // toDataURL must be supported because the link to this
  // function is disabled if it is not.
  var dataurl = canvas.toDataURL();
  $("#c1").hide();
  $("#s1").html("<img src='"+dataurl+"'>").show();
  $("#saveInstructions").slideDown('slow');
}

function enableDrawing()
{
  if ($("#c1").css('display') == 'none')
  {
    $("#saveInstructions").slideUp('slow');
    $("#s1").hide();
    $("#c1").show();
  }
}

var numberOfGenerations;
var moduleCount;
var lastls = null; // remembers the last lsystem incase it needs to be redrawn on resize
var lastStyle;
var lastWidth;
var lastBg;
var strDrawCurGen = "Draw current generation";

function saveLast(ls, t)
{
  lastls = ls;
  lastStyle = t.penStyle;
  lastWidth = t.penWidth;
}

function renderDone(ls)
{
  if (ls)
  {
    ls.elapsedTime = (new Date()).getTime() - ls.elapsedTime;
  }
  rendering = null;
  $("#whichLsys").attr("disabled","");
  $("#play-stop").attr('title', strDrawCurGen).find('img').attr(
  					'src','res/i/play.gif').attr('alt', strDrawCurGen);
  $("#next").add("#prev").attr("disabled", "");
  progressHide();
  // update info
  if (ls)
  {
    updateInfo(ls);
  }
}

// pattern for breaking up long running task into chuncks described here:
// http://www.julienlecomte.net/blog/2007/10/28/
function render(ls, t)
{
  // Init
  var style = getPenColor();
  var width = getPenWidth();
  clearCanvas();
  t.style(style);
  t.width(width);
  t.penDown();
  if (ls.beforeRender)
  {
    ls.beforeRender();
  }
  ls.renderBegin();
  moduleCount = ls.moduleCount;
  var steps = Math.ceil(ls.moduleCount / getModulesPerStep());
  progressStart("#renderProgress", steps);

  (function () {
    var start;
    var end;
    try
    {
      // check if done
      if (!ls.renderHasNext())
      {
        // done
        saveLast(ls, t);
        renderDone(ls);
        return;
      }
      // process one chunck
      start = (new Date()).getTime();
      ls.renderNextN(getModulesPerStep());
      end = (new Date()).getTime();
      ls.renderTime += end - start;
      progressStep("#renderProgress");
      // Process next chunk
      rendering = setTimeout(arguments.callee, getDelay());
    }
    catch (ex)
    {
      alert("Internal error", ex);
      renderDone(ls);
      throw ex;
    }
  })();
}

// pattern for breaking up long running task into chuncks described here:
// http://www.julienlecomte.net/blog/2007/10/28/
function generate(ls, t)
{
  // Init
  if (ls.generation < numberOfGenerations)
  {
    ls.generateBegin();
    var steps = numberOfGenerations;
    progressStart("#genProgress", steps);
  }
  else
  {
    // done - nothing to generate
    render(ls, t);
    return;
  }

  (function () {
    try
    {
      // check if done
      if (!ls.generateHasNext())
      {
        progressStep("#genProgress");
        if (ls.generation >= numberOfGenerations)
        {
          // done
          render(ls, t);
          return;
        }
        ls.generateBegin();
      }
      // process one chunck
      var start = (new Date()).getTime();
      ls.generateNextN(1000);
      var end = (new Date()).getTime();
      ls.generateTime += end - start;
      // Process next chunk
      rendering = setTimeout(arguments.callee, 0);
    }
    catch (ex)
    {
      alert("Internal error", ex);
      renderDone(ls);
      throw ex;
    }
  })();
}

function generateAndRender()
{
  enableDrawing();
  lastls = null;
  progressShow();
  progressStart("#renderProgress", 1);
  progressStart("#genProgress", 1);
  var which = $("#whichLsys").attr("disabled","disabled").val();
  $("#play-stop").attr('title', "Cancel drawing").find('img').attr('src','res/i/stop.gif');
  $("#next").add("#prev").attr("disabled", "disabled");

  var ex = examples[which];
  t.home();
  var ls = ex(t);
  ls.renderTime = 0;
  ls.generateTime = 0;
  ls.elapsedTime = (new Date()).getTime();
  numberOfGenerations = getNumGen();
  generate(ls, t);
}

function reRender()
{
  enableDrawing();
  var which = $("#whichLsys").val();
  var ex = examples[which];
  // need a new lsystem so that the turtle context is updated
  t.home();
  var ls = ex(t);
  // but avoid regeneration by using the curgen from the last lsystem that was fully rendered.
  ls.curgen = lastls.curgen;
  ls.moduleCount = lastls.moduleCount;
  ls.generation = lastls.generation;
  // Init
  t.clear(lastBg);
  t.style(lastStyle);
  t.width(lastWidth);
  t.penDown();
  if (ls.beforeRender)
  {
    ls.beforeRender();
  }
  ls.render(); // this will block the UI but print preview doesn't seem to allow using the timer.
}

$(document).ready(function() {
    var options = "";
    var dialogOptions = {
        height: 300,
        width: "30em",
        resizeable: true,
        dragable: true,
        position: "center",
        buttons: {"OK": function() {$(this).dialogClose();} }
    };

    $("#bd").show();

    // $.ajaxHistory.initialize(); xxx todo: not working

    $("#c1").replaceWith('<canvas id="c1" width="460" height="460">' +
      'This browser doesn\'t support the canvas feature needed by this page.' + 
      'Try a recent version of <a href="http://www.mozilla.com/en-US/firefox/">Firefox<\/a><\/canvas>');
    canvas = $("#c1").get(0);

    // for each of the model divs hide it and get the select option value and label
    $("#models > div").hide().each(function () {
      var value = $(this).attr('id');
      var label = $(this).find("h2").text();
      options += "<option value='" + value + "'>" + label + "<\/option>";
    });

    $("#whichLsys").html(options);
    $("#whichLsys option:first").attr("selected", "selected");

    // sliders affect progress so init progress ctrls first
    progressInit();

    // setup sliders
    $("#modulesPerStep").slider({minValue: 1, maxValue: 800, startValue: 100 });
    $("#delay").slider({minValue: 0, maxValue: 1000, startValue: 50 });
    // bind a handler to update the numeric display also
    // Add <p> element to hold the current value and tabindex attribute so it can take focus
    $(".ui-slider-1").attr("tabindex", "0").after("<p></p>").bind("slide", function(e, ui) {
      // xxx work around a jQuery bug? in slider when the event data is prepared e is null
      // and in jquery where the even is dispatched a fake event is created and prepended
      // to the data but this pushes the ui arg down one.
      if (ui === null)
      {
        ui = arguments[2];
      }
      $(this).next("p").html("" + ui.value);
      if (moduleCount && $(this).attr("id") == "modulesPerStep")
      {
        var steps = Math.ceil(moduleCount / ui.value);
        progressSetMax("#renderProgress", steps);
      }
    }).each(function () { // and bind keyboard support
		    var sliderEl = $(this);
		    var slider = sliderEl.sliderInstance();
        var o = slider.interaction.options;
		    $(this).bind("keydown", function (e) {
	  			switch(e.keyCode) {
		  		case 37:
              slider.moveTo(slider.interaction.curValue + o.minValue - 10, 0, true);
				  		break;
  				case 39:
	  					slider.moveTo(slider.interaction.curValue + o.minValue + 10, 0, true);
		  				break;
			  	}
				  if(e.keyCode != 9) {
				    return false;
				  }
				  return true;
        });
        // do this to get the initial numeric display
        slider.moveTo(slider.interaction.curValue + o.minValue, 0, true);
    });

    // for text inputs make them something we can style
    $("input:text").addClass("text");

    if ($.browser.msie)
    {
      canvas = G_vmlCanvasManager.initElement(canvas);
    }

    dialogOptions.title = "About L-system.html";
    $("#about").dialogInit(dialogOptions);
    dialogOptions.height = 400;
    dialogOptions.title = "Info";
    $("#info").dialogInit(dialogOptions);

    $("#aboutCmd").click(function() {
      $("#about").dialog();
      return false;
    });

    clearInfo();
    $("#infoCmd").click(function() {
      $("#info").dialog();
      return false;
    });

    if (canvas.toDataURL)
    {
      $("#saveCmd").click(function() {
        save();
        return false;
      });
    }
    else
    {
      $("#saveCmd").hide();
    }

    if (!(canvas && canvas.getContext))
    {
      return; // canvas is not supported so no point in going on
    }

    // Handle window resize. Make the canvas larger or smaller. Redraw as needed.
    $(window).resize(function() {
      var width = $("#drawingArea").width()-2; // - 2 for border
      console.log("On resize: new=" + width + ", old=" + canvas.width);
      if (width != canvas.width)
      {
        if ($.browser.msie)
        {
          canvas = G_vmlCanvasManager.initElement(canvas);
        }
        canvas.width = width;
        jjs.Turtle.init(canvas);
        t = new jjs.Turtle(canvas);
        if (lastls)
        {
          reRender();
        }
      }
    }).trigger('resize');

    // handler for when whichLsys selection changes
    $("#whichLsys").change(function(e) {
      var model = e.target.value;
      $("#models > div").hide('slow').find("input").attr("disabled","disabled");
      var heading = $("#" + model).show('slow').find("input").attr("disabled", "").end().find("h2").text();
      $("#drawingArea h2").text(heading);
      /* xxx todo: This history stuff isn't working
      location.hash = model;
      $.ajaxHistory.update(model); */
      if (examples[model])
      {
        $("#playerButtons, #globalSettings").show();
      }
      else
      {
        $("#playerButtons, #globalSettings").hide();
      }
      clearInfo();
      clearCanvas();
      lastls = null;
    }).change(); // and fire it the first time

    $("#play-stop").click(function(e) {
      if (rendering === null)
      {
        if (allFieldsValid())
        {
          try
          {
            generateAndRender();
          }
          catch (ex)
          {
            alert("Internal error:", ex);
            renderDone();
            throw ex;
          }
        }
      }
      else
      {
        // cancel
        clearTimeout(rendering);
        renderDone();
      }
    }).attr('title', strDrawCurGen).find('img').attr('alt', strDrawCurGen);
    $("#next").click(function(e) {
      if (allFieldsValid())
      {
        // inc generation
        incNextGen();
        generateAndRender();
      }
    });
    $("#prev").click(function(e) {
      if (allFieldsValid())
      {
        // dec generation
        decNextGen();
        generateAndRender();
      }
    });

    // attach validation rules to each control
    for (id in validations) if (validations.hasOwnProperty(id))
    {
      $("#"+id).each(function() {
        $(this).change(function(){validateField(this);});
        validateField(this);
      });
    }

});

})();
