<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
	<title>L-System tests</title>
	<meta http-equiv="content-type" content="text/html;charset=utf-8">
	<meta http-equiv="Content-Style-Type" content="text/css">
  $resources.lsystemlib.src: {<script src="../$it$" type="text/javascript"></script>
  }$
	<script src="../lib/lsystem.js" type="text/javascript"></script>
	<script src="../res/js/jquery-1.2.2.js" type="text/javascript"></script>
	<script src="jsUnitCore.js" type="text/javascript"></script>
  <script type="text/javascript" src="../res/firebug/firebug.js"></script>

<script type="text/javascript">

function setUp()
{
}

function tearDown()
{
}

// test TreeIterator first
function testTreeIterator()
{
  console.log("test tree iterator");

  var t = new TreeIterator([ ]);

  assertFalse(t.hasNext());
  try {
    t.next();
    fail();
  }
  catch (ex)
  {
    assertEquals("No next.", ex);
  }

  t = new TreeIterator([ {id: "A"} ]);
  assertTrue(t.hasNext());
  assertTrue(t.hasNext());
  assertEquals("A", t.next().id);
  assertFalse(t.hasNext());
  assertFalse(t.hasNext());
  try {
    t.next();
    fail();
  }
  catch (ex)
  {
    assertEquals("No next.", ex);
  }

  t = new TreeIterator([ {id: "A"}, {id: "B"} ]);
  assertTrue(t.hasNext());
  assertEquals("A", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("B", t.next().id);
  assertFalse(t.hasNext());

  t = new TreeIterator([ {id: "A"}, [{id: "B"}, {id: "C"}], {id: "D"}]);
  assertTrue(t.hasNext());
  assertEquals("A", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("B", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("C", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertTrue(t.hasNext());
  assertEquals("D", t.next().id);
  assertFalse(t.hasNext());

  t = new TreeIterator([ {id: "A"}, [ [{id: "B"}], {id: "C"}], {id: "D"}]);
  assertTrue(t.hasNext());
  assertEquals("A", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("B", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertTrue(t.hasNext());
  assertEquals("C", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertTrue(t.hasNext());
  assertEquals("D", t.next().id);
  assertFalse(t.hasNext());

  t = new TreeIterator([ [ [{id: "B"} ] ] ]);
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("B", t.next().id);
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertFalse(t.hasNext());
  try {
    t.next();
    fail();
  }
  catch (ex)
  {
    assertEquals("No next.", ex);
  }

  t = new TreeIterator([ [ [ ] ] ]);
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertFalse(t.hasNext());
}

// test TreeIterator context
function testTreeIteratorContext()
{
  console.log("test tree iterator context");

  var t = new TreeIterator(
    [ {id:"A"},{id:"B"},{id:"C"},[{id:"D"},{id:"D1"}],[{id:"E"},{id:"E1"},[{id:"X"}],{id:"E2"}],[{id:"F"},{id:"F1"}],
      {id:"G"},[{id:"H"}],{id:"I"},{id:"J"},{id:"K"} ]);

  function matcher(s)
  {
    return function (m) {
      if (s === "*")
      {
        return true;
      }
      return m.id === s;
    };
  }

  // A
  assertTrue(t.hasNext());
  assertEquals("A", t.next().id);
  assertNull(t.previousSiblingOrParent());
  t.resetContext();
  assertNull(t.previousSibling());
  t.resetContext();
  assertNull(t.parent());
  t.resetContext();
  assertEquals("B", t.nextSibling().id);
  assertEquals("C", t.nextSibling().id);
  assertEquals("G", t.nextSibling().id);
  t.resetContext();
  assertNull(t.findChild(matcher("B")));
  t.resetContext();
  assertNull(t.findChild(matcher("E")));
  t.resetContext();
  assertNull(t.findChild(matcher("G")));

  // B
  assertTrue(t.hasNext());
  assertEquals("B", t.next().id);
  assertEquals("A", t.previousSiblingOrParent().id);
  assertNull(t.previousSiblingOrParent());
  t.resetContext();
  assertEquals("A", t.previousSibling().id);
  assertNull(t.previousSibling());
  t.resetContext();
  assertNull(t.parent());
  t.resetContext();
  assertEquals("C", t.nextSibling().id);
  assertEquals("G", t.nextSibling().id);
  assertEquals("I", t.nextSibling().id);
  t.resetContext();
  assertNull(t.findChild(matcher("B")));

  // C
  assertTrue(t.hasNext());
  assertEquals("C", t.next().id);
  assertEquals("B", t.previousSiblingOrParent().id);
  assertEquals("A", t.previousSiblingOrParent().id);
  t.resetContext();
  assertEquals("B", t.previousSibling().id);
  assertEquals("A", t.previousSibling().id);
  t.resetContext();
  assertNull(t.parent());
  t.resetContext();
  assertEquals("G", t.nextSibling().id);
  assertEquals("I", t.nextSibling().id);
  assertEquals("J", t.nextSibling().id);
  t.resetContext();
  assertEquals("E", t.findChild(matcher("E")).id);
  assertEquals("E1", t.nextSibling().id);
  t.resetContext();
  assertNull(t.findChild(matcher("Q")));
  t.resetContext();
  assertEquals("D", t.findChild(matcher("D")).id);
  assertEquals("D1", t.nextSibling().id);
  t.resetContext();
  assertEquals("F", t.findChild(matcher("F")).id);
  assertEquals("C", t.parent().id);
  t.resetContext();
  assertEquals("G", t.findChild(matcher("G")).id);
  assertEquals("H", t.findChild(matcher("H")).id);

  // D
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("D", t.next().id);
  assertEquals("C", t.previousSiblingOrParent().id);
  assertEquals("B", t.previousSiblingOrParent().id);
  t.resetContext();
  assertNull(t.previousSibling());
  t.resetContext();
  assertEquals("C", t.parent().id);
  t.resetContext();
  assertEquals("D1", t.nextSibling().id);
  assertNull(t.nextSibling());
  t.resetContext();
  assertNull(t.findChild(matcher("*")));

  // D1
  assertTrue(t.hasNext());
  assertEquals("D1", t.next().id);
  assertEquals("D", t.previousSiblingOrParent().id);
  assertEquals("C", t.previousSiblingOrParent().id);
  t.resetContext();
  assertEquals("D", t.previousSibling().id);
  assertNull(t.previousSibling());
  t.resetContext();
  assertNull(t.parent());
  t.resetContext();
  assertNull(t.nextSibling());
  t.resetContext();
  assertNull(t.findChild(matcher("*")));

  // E
  assertTrue(t.hasNext());
  assertEquals("]", t.next());
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("E", t.next().id);
  assertEquals("C", t.previousSiblingOrParent().id);
  assertEquals("B", t.previousSiblingOrParent().id);
  t.resetContext();
  assertNull(t.previousSibling());
  t.resetContext();
  assertEquals("C", t.parent().id);
  t.resetContext();
  assertEquals("E1", t.nextSibling().id);
  t.resetContext();
  assertNull(t.findChild(matcher("E1")));

  // E1
  assertTrue(t.hasNext());
  assertEquals("E1", t.next().id);
  assertEquals("E", t.previousSiblingOrParent().id);
  assertEquals("C", t.previousSiblingOrParent().id);
  t.resetContext();
  assertEquals("E", t.previousSibling().id);
  t.resetContext();
  assertNull(t.parent());
  t.resetContext();
  assertEquals("E2", t.nextSibling().id);
  t.resetContext();
  assertEquals("X", t.findChild(matcher("X")).id);

  // X
  assertTrue(t.hasNext());
  assertEquals("[", t.next());
  assertTrue(t.hasNext());
  assertEquals("X", t.next().id);
  assertEquals("E1", t.previousSiblingOrParent().id);
  t.resetContext();
  assertNull(t.previousSibling());
  t.resetContext();
  t.resetContext(); // should cause no problem
  assertEquals("E1", t.parent().id);
  t.resetContext();
  assertEquals("E1", t.previousSiblingOrParent().id);
  assertEquals("X", t.findChild(matcher("X")).id);
  assertEquals("E1", t.parent().id);
}

// test a deterministic context-free lsystem
// covers basics of generate and render
// A, B
// A
// A --> AB
// B --> A
function testDetCF()
{
  console.log("deterministic context-free non-parametric lsystem");
  var r;
  var ls = new jjs.Lsystem({
                        A: function() { r += "do a,"; },  // a simple renderer
                        B: function() { r += "do b,"; }
                      }, 
                      [ {id: "A"} ],  // a single initial module
                      [ 
                        { p: {id: "A"}, // list is optional here
                          // condition is optional
                          s: function() { return [{id:"A"},{id:"B"}];} }, // list is optional here
                        { p: [ {id: "B"} ],
                          s: function() { return [{id:"A"}];} }
                      ]
                      // ignore is optional and empty here
                      );
  assertEquals(0, ls.generation);
  assertUndefined(ls.ignore);
  assertEquals(1, ls.moduleCount);
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[A]", ls.toString());
  ls.generate(); // test next
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals(1, ls.generation);
  assertEquals(2, ls.moduleCount);
  assertEquals("[A, B]", ls.toString());
  r = "";
  ls.render();
  assertEquals("do a,do b,", r);
  ls.generate();
  assertEquals(2, ls.generation);
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[A, B, A]", ls.toString());
  ls.reset(); // test reset
  assertEquals(0, ls.generation);
  assertEquals("[A]", ls.toString());
  ls.generateN(3); // test nextN
  assertEquals("[A, B, A, A, B]", ls.toString());
  assertEquals(3, ls.generation);
  // test other render methods begin, hasnext, next, nextn
  r = "";
  ls.renderBegin();
  assertTrue(ls.renderHasNext());
  ls.renderNext();
  assertEquals("do a,", r);
  assertTrue(ls.renderHasNext());
  ls.renderNext();
  assertEquals("do a,do b,", r);
  assertTrue(ls.renderHasNext());
  assertEquals(2, ls.renderNextN(2));
  assertTrue(ls.renderHasNext());
  assertEquals(1, ls.renderNextN(3));
  assertFalse(ls.renderHasNext());
  try {
    ls.renderNext();
    fail();
  }
  catch (e)
  {
    assertEquals("No next.", e);
  }
  // reset again and try iterative generate methods
  ls.reset();
  assertEquals(0, ls.generation);
  assertEquals(1, ls.moduleCount);
  ls.generateBegin();
  assertTrue(ls.generateHasNext());
  ls.generateNext();
  assertFalse(ls.generateHasNext());
  try {
    ls.generateNext();
    fail();
  }
  catch (e)
  {
    assertEquals("No next.", e);
  }
  assertEquals(1, ls.generation);
  assertEquals(2, ls.moduleCount);
  assertEquals("[A, B]", ls.toString());
  ls.generateBegin();
  assertTrue(ls.generateHasNext());
  ls.generateNext();
  assertEquals(1, ls.generation); // still 1
  assertTrue(ls.generateHasNext());
  ls.generateNext();
  assertFalse(ls.generateHasNext());
  assertEquals(2, ls.generation);
  assertEquals(3, ls.moduleCount);
  assertEquals("[A, B, A]", ls.toString());
  ls.generateBegin();
  assertTrue(ls.generateHasNext());
  assertEquals(3, ls.generateNextN(27));
  assertEquals(3, ls.generation);
  assertEquals(5, ls.moduleCount);
  assertEquals("[A, B, A, A, B]", ls.toString());

}

// test a deterministic context-free parametric lsystem
// A, B
// A(1)
// A(x) --> x >= 3 ? A(x)B
// A(x) --> A(x + 1)B
// B --> A(1)
function testDetCFPara()
{
  console.log("deterministic context-free parametric lsystem");
  var ls = new jjs.Lsystem({ }, 
                       [ {id: "A", args:[1]} ], 
                       [ 
                         { p: [ {id: "A", a: 1} ], // here the list is explicit
                           condition: function(x) { return x >= 3; },
                           // optional list explicit here the 
                           s: [ { p: 1, f: function(x) { return [{id: "A", args: [x]},{id:"B"}];} }]},
                         { p: [ {id: "A", a: 1} ], 
                           s: function(x) { return [{id: "A", args: [x + 1]},{id:"B"}];} },
                         { p: [ {id: "B"} ],
                           s: function() { return [{id: "A", args: [1]}];} }
                       ]
                      );
  assertEquals("[A(1)]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[A(2), B]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[A(3), B, A(1)]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[A(3), B, A(1), A(2), B]", ls.toString());

}

// test the same deterministic context-free parametric lsystem as above
// but this time use symbols and multi-character ids
// *, b1
// *(1)
// *(x) --> x >= 3 ? *(x)b1
// *(x) --> *(x + 1)b1
// b1 --> *(1)
function testNonCharIds()
{
  console.log("test symbols with symbols and multiple characters");
  var ls = new jjs.Lsystem({ 
                        "*": function(x) { this.r += "do A(" + x + "),"; },
                        "b1": function() { this.r += "do B,"; }
                       }, 
                       [ {id: "*", args:[1]} ], 
                       [ 
                         { p: [ {id: "*", a: 1} ], 
                           condition: function(x) { return x >= 3; },
                           s: function(x) { return [{id: "*", args: [x]},{id:"b1"}];} },
                         { p: [ {id: "*", a: 1} ], 
                           s: function(x) { return [{id: "*", args: [x + 1]},{id:"b1"}];} },
                         { p: [ {id: "b1"} ],
                           s: function() { return [{id: "*", args: [1]}];} }
                       ]
                      );
  ls.r = "";
  assertEquals("[*(1)]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[*(2), b1]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[*(3), b1, *(1)]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[*(3), b1, *(1), *(2), b1]", ls.toString());
  ls.r = "";
  var out = ls.render();
  console.log(ls.r);
  assertEquals("do A(3),do B,do A(1),do A(2),do B,", ls.r);
}

// test a branching
function testBranching()
{
  console.log("test branching");
  // n=7,δ=20deg
  // X
  // X-->F[+X]F[-X]+X
  // F-->FF
  var ls = new jjs.Lsystem({ 
                        "X": function() { this.r += "f(n),"; },
                        "F": function() { this.r += "f(n),"; },
                        "+": function() { this.r += "l(20),"; },
                        "-": function() { this.r += "r(20),"; },
                        "[": function() { this.r += "push,"; },
                        "]": function() { this.r += "pop,"; }
                       }, 
                       [ {id: "X"} ], 
                       [ 
                         { p: [ {id: "X"} ], 
                           s: function() { return [{id: "F"},[{id:"+"},{id:"X"}], 
                                                    {id: "F"},[{id:"-"},{id:"X"}],{id:"+"},{id:"X"}];} },
                         { p: [ {id: "F"} ],
                           s: function() { return [{id: "F"},{id:"F"}];} }
                       ]
                      );
  ls.r = "";
  assertEquals("[X]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  assertEquals("[F, [+, X], F, [-, X], +, X]", ls.toString());
  assertEquals(12, ls.moduleCount);
  ls.render();
  assertEquals("f(n),push,l(20),f(n),pop,f(n),push,r(20),f(n),pop,l(20),f(n),", ls.r);
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  assertEquals("[F, F, [+, F, [+, X], F, [-, X], +, X], F, F, [-, F, [+, X], F, [-, X], +, X], +, F, [+, X], F, [-, X], +, X]", 
    ls.toString());
}

// test a branching but this time flat structure
// n=7,δ=20deg
// X
// X-->F[+X]F[-X]+X
// F-->FF

function testBranching2()
{
  console.log("test branching no sub arrays");
  var ls = new jjs.Lsystem({ 
                        "X": function() { this.r += "f(n),"; },
                        "F": function() { this.r += "f(n),"; },
                        "+": function() { this.r += "l(20),"; },
                        "-": function() { this.r += "r(20),"; },
                        "[": function() { this.r += "push,"; },
                        "]": function() { this.r += "pop,"; }
                       }, 
                       [ {id: "X"} ], 
                       [ 
                         { p: [ {id: "X"} ], 
                           s: function() { return [{id: "F"},{id:"["},{id:"+"},{id:"X"},{id:"]"}, 
                                                    {id: "F"},{id:"["},{id:"-"},{id:"X"},{id:"]"},{id:"+"},{id:"X"}];} },
                         { p: [ {id: "F"} ],
                           s: function() { return [{id: "F"},{id:"F"}];} }
                       ]
                      );
  ls.r = "";
  assertEquals("[X]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  assertEquals("[F, [+, X], F, [-, X], +, X]", ls.toString());
  assertEquals(12, ls.moduleCount);
  ls.render();
  assertEquals("f(n),push,l(20),f(n),pop,f(n),push,r(20),f(n),pop,l(20),f(n),", ls.r);
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  assertEquals("[F, F, [+, F, [+, X], F, [-, X], +, X], F, F, [-, F, [+, X], F, [-, X], +, X], +, F, [+, X], F, [-, X], +, X]", 
    ls.toString());
}

// test delete rule in a branching L-system
function testDelete()
{
  console.log("test delete");
  // δ=20deg
  // X
  // X-->F(0)[-X]
  // F(a)-->F(a+1)
  // F(a)-->(a>=2)
  var ls = new jjs.Lsystem({ }, 
                       [ {id: "X"} ], 
                       [ 
                         { p: [ {id: "X"} ], 
                           s: function() { return [{id: "F", args:[0]},[{id:"+"},{id:"X"}]];} },
                         { p: [ {id: "F", a:1} ],
                           s: function(a) { return [{id:"F", args:[a+1]}];} },
                         { p: [ {id: "F", a:1} ],
                           condition: function(a) { return a >=2; },
                           s: function(a) { return [];} } // delete the symbol
                       ]
                      );
  ls.r = "";
  assertEquals("[X]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls.curgenToString(", "));
  assertEquals("[[+, F(2), [+, F(1), [+, F(0), [+, X]]]]]", ls.toString());
}

function testInitErrors()
{
  console.log("init errors");
  var r;
  var ls;

  // test that rules are fixed up correctly
  ls = new jjs.Lsystem({ }, 
                    [ {id: "A"} ], 
                    [ 
                      { p: [ { id: "A"} ], 
                        s: function() { return [{id:"A"},{id:"B"}];} },
                      { p: [ {id: "B"} ],
                        s: function() { return [{id:"A"}];} }
                    ]
                    );
  assertEquals(0, ls.rules[0].p[0].a);
  assertEquals(0, ls.rules[1].p[0].a);
  assertEquals("object", typeof ls.rules[0].s);
  assertEquals("function", typeof ls.rules[0].s[0].f);
  assertEquals(1, ls.rules[0].s[0].p);
  

  // test error handeling
  try {
    ls = new jjs.Lsystem({ }, 
                      [ {id: "A"} ], 
                      [ 
                        { p: [ { dir: "<", id: "A"} ], 
                          s: function() { return [{id:"A"},{id:"B"}];} },
                        { p: [ {id: "B"} ],
                          s: function() { return [{id:"A"}];} }
                      ]
                      );
    fail();
  }
  catch (e)
  {
    assertEquals("Strict predecessor must not specify a direction.", e);
  }
  try {
    ls = new jjs.Lsystem({ }, 
                      [ {id: "A"} ], 
                      [ 
                        { p: [ {id: "B", a: 4}, { id: "A"} ], 
                          s: function() { return [{id:"A"},{id:"B"}];} },
                        { p: [ {id: "B"} ],
                          s: function() { return [{id:"A"}];} }
                      ]
                      );
    fail();
  }
  catch (e)
  {
    assertEquals("Context predecessor must specify a direction.", e);
  }
  try {
    ls = new jjs.Lsystem({ }, 
                      [ {id: "A"} ], 
                      [ 
                        { p: [ {id: "B", a: 4}, { dir: "<", id: "A"} ], 
                          s: function() { return [{id:"A"},{id:"B"}];} },
                        { p: [ {id: "B"} ],
                          s: [ {p: 0.5, f: function() { return [{id:"A"}];} },
                               {p: 0.2, f: function() { return [{id:"B"}];} } ] }
                      ]
                      );
    fail();
  }
  catch (e)
  {
    assertEquals("Probabilities must sum to 1.", e);
  }
 
}

// xxx not used
function testAccess()
{
  console.log("access tests");
  ls = new jjs.Lsystem({ }, 
                      [ {id: "A"} ], 
                      [ 
                        { p: [ {id: "B", a: 4} ], 
                          s: function() { return [{id:"A"},{id:"B"}];} },
                        { p: [ {id: "B"} ],
                          s: function() { return [{id:"A"}];} }
                      ]
                      );
  assertUndefined(ls.renderIterator);
  assertUndefined(ls.init);
  assertNotUndefined(ls.generate);
}

// because there is no seed function it is hard to test
// random rule selection. This is as much a test of the
// uniform distribution of the random function as the Lsystem code.
function testNonDeterministic()
{
  var xcount = 0;
  var ycount = 0;
  var totalcount = 0;
  console.log("non-deterministic");
  ls = new jjs.Lsystem({ X: function() { xcount++; totalcount++; },
                         Y: function() { ycount++; totalcount++; }
                       }, 
                      [ {id: "A"},{id: "A"},{id: "A"},{id: "A"},{id: "A"},
                        {id: "A"},{id: "A"},{id: "A"},{id: "A"},{id: "A"} ], 
                      [ 
                        { p: [ {id: "A"} ], 
                          s: function() { return [{id:"B"},{id:"B"},{id:"B"},{id:"B"},{id:"B"},
                                                  {id:"B"},{id:"B"},{id:"B"},{id:"B"},{id:"B"}];} },
                        { p: [ {id: "B"} ], 
                          s: function() { return [{id:"C"},{id:"C"},{id:"C"},{id:"C"},{id:"C"},
                                                  {id:"C"},{id:"C"},{id:"C"},{id:"C"},{id:"C"}];} },
                        { p: [ {id: "C"} ], 
                          s: function() { return [{id:"D"},{id:"D"},{id:"D"},{id:"D"},{id:"D"},
                                                  {id:"D"},{id:"D"},{id:"D"},{id:"D"},{id:"D"}];} },
                        { p: [ {id: "D"} ], 
                          s: [ {p: 0.7, f: function() { return [{id:"X"}];} },
                               {p: 0.3, f: function() { return [{id:"Y"}];} } ] }
                      ]
                      );
  ls.generateN(4);
  assertEquals(4, ls.generation);
  assertEquals(10000, ls.curgen.length);
  ls.render();
  console.log("x: " + xcount + " y: " + ycount + " x/y: " + xcount/ycount);
  assertTrue(xcount/ycount < 3 && xcount/ycount > 2);
}

// test a context-sensitive L-system
function testContextSensitive()
{
  console.log("context-sensitive non-parametric lsystem");
  // b, a, a, a, a
  // a <b --> b
  // b --> a
  var ls = new jjs.Lsystem({ }, 
                       [ {id: "b"},{id: "a"},{id: "a"},{id: "a"},{id: "a"} ], 
                       [ 
                         { p: [ {id: "a"}, { dir: "<", id: "b"}],
                           s: function() { return [{id: "b"}];} },
                         { p: [ {id: "b"} ], 
                           s: function() { return [{id: "a"}];} }
                       ]
                      );
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[b, a, a, a, a]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a, b, a, a, a]", ls.toString());
  ls.generateN(3);
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a, a, a, a, b]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a, a, a, a, a]", ls.toString());

  // test multiple rules with same strict pred and both < and >
  // b a b
  // c < a > c --> b a b
  // b < a > b --> c a c
  ls = new jjs.Lsystem({ }, 
                       [ {id: "b"},{id: "a"},{id: "b"} ], 
                       [ 
                         { p: [ {id: "a"}, { dir: "<", id: "c"}, { dir: ".>", id: "c"}],
                           s: function() { return [{id: "b"},{id: "a"},{id: "b"}];} },
                         { p: [ {id: "a"}, { dir: "<", id: "b"}, { dir: ".>", id: "b"} ], 
                           s: function() { return [{id: "c"},{id: "a"},{id: "c"}];} }
                       ]
                      );
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[b, a, b]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[b, c, a, c, b]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[b, c, b, a, b, c, b]", ls.toString());

  //xxx test other directions and multiple steps in each direction and ignore
}

// test for proper order in context-sensitive, parametric L-system
// a(0)
// a(x) --> a(x+1) b
// a(x) > b --> a(x + 1) c
// a(x) --> (x >= 3) a(x), a(0)
// rules get matched in reverse order from given above
function testRuleOrder()
{
  console.log("rule order");
  var ls = new jjs.Lsystem({ }, 
                       [ {id: "a", args: [0]} ], 
                       [ 
                         { p: {id: "a", a: 1},
                           s: function(x) { return [{id: "a", args: [x + 1]}, {id: "b"}];} },
                         { p: [ {id: "a", a: 1}, { dir: ">", id: "b"}],
                           s: function(x) { return [{id: "a", args: [x + 1]}, {id: "c"}];} },
                         { p: [ {id: "a", a: 1} ],
                           condition: function(x) { return x >= 3; },
                           s: function(x) { return [{id: "a", args: [x]},{id: "a", args: [0]}];} }
                       ]
                      );
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a(0)]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a(1), b]", ls.toString());
  ls.generateN(1);
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a(2), c, b]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a(3), b, c, b]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a(3), a(0), b, c, b]", ls.toString());
  ls.generate();
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a(3), a(0), a(1), c, b, c, b]", ls.toString());
}

function testInternal()
{
  console.log("internal tests");
  var val;
  var ls = new jjs.Lsystem({ }, 
                       [ {id: "a"} ], 
                       [ 
                         { p: {id: "a"},
                           s: function(x) { return [{id: "a"}, {id: "b"}];} }
                       ]
                      );
  console.log("gen: " + ls.generation + " is: " + ls);
  assertEquals("[a]", ls.toString());

  val = 5;
  assertEquals("5", ls.valueToString(val));
  val = "five";
  assertEquals("five", ls.valueToString(val));
  val = {x: 7, y: 8};
  assertEquals("{x: 7, y: 8}", ls.valueToString(val));
  val = ["a", 4, true];
  assertEquals('[a, 4, true]', ls.valueToString(val));
  console.log(ls.valueToString(val));
}

function runTests()
{
  \$.each([testTreeIterator, testTreeIteratorContext, testDetCF,
          testDetCFPara, testNonCharIds, testBranching, testBranching2, 
          testDelete,
          testInitErrors, testNonDeterministic, testContextSensitive,
          testRuleOrder, testInternal]
         , function() {
         setUp();
         this();
         tearDown();
         });
}

\$(document).ready(function() {
  \$("#runtests").click(runTests);
});


</script>
</head>
<body>
	<h1>L-system Testing</h1>
	<p>This page contains tests for the lsystem.js module. <input type="button" value="run" id="runtests"></p>
</body>
</html>
