<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
	<title>Turtle tests</title>
	<meta http-equiv="content-type" content="text/html;charset=utf-8">
	<meta http-equiv="Content-Style-Type" content="text/css">
	<!--[if IE]><script type="text/javascript" src="../res/js/excanvas.js"></script><![endif]-->
  $resources.turtlelib.src: {<script src="../$it$" type="text/javascript"></script>
  }$
	<script src="../res/js/jquery-1.2.2.js" type="text/javascript"></script>
	<script src="jsUnitCore.js" type="text/javascript"></script>
  <script type="text/javascript" src="../res/firebug/firebug.js"></script>

<style type="text/css">
#c1 {
  border: 1px solid green;
  background-color: #FEFF6F;
}
</style>
<script type="text/javascript">

var theCanvas = null;

function setUp()
{
  \$("#canvasHolder").html("<canvas id=\"c1\" width=\"250\" height=\"250\"><\/canvas>");
  theCanvas = \$("#c1").get(0);
  // special case for IE
  if (window.G_vmlCanvasManager)
  {
    theCanvas = G_vmlCanvasManager.initElement(theCanvas);
  }
}

function tearDown()
{
  \$("#canvasHolder").empty();
  var theCanvas = null;
}

function testInit()
{
  assertNotNull("The canvas must have been set up.", theCanvas);
  assertNotUndefined("Canvas context must be supported.", theCanvas.getContext);

  // this is to verify that the initial turtle settings are correct
  // and that the init method moves the origin to the center and makes
  // positive y values go up the screen rather than down.
  var ctx = theCanvas.getContext('2d');
  ctx.lineWidth = 4;
  ctx.strokeStyle = 'blue';
  ctx.beginPath();
  ctx.moveTo(0,-10);
  ctx.lineTo(0,10);
  ctx.moveTo(-10,0);
  ctx.lineTo(10, 0);
  ctx.moveTo(-10, -18);
  ctx.lineTo(10, -18);
  ctx.stroke();
  jjs.Turtle.init(theCanvas);
  var t = new jjs.Turtle(theCanvas);
  assertEquals(t.x, 0.0);
  assertEquals(t.y, 0.0);
  assertEquals(t.h, 0.0);
  assertFalse(t.penIsDown);
  assertEquals(t.stack.length, 0);
  assertEquals(t.penWidth, 2);
  assertEquals(t.penStyle, "black");
  assertEquals(t.toString(), "{x: 0, y: 0, pen: up, penWidth: 2, penStyle: black, stack depth: 0}");
  t.style('red').width(4);
  t.forward(10).left(180);
  t.penDown().forward(20).penUp().right(180).forward(10).left(90).forward(10).right(180).penDown().forward(20);
  t.penUp().forward(8).left(90).forward(10).right(180).penDown().forward(20);

  assertTrue(confirm("Do you see 1/4 of an blue + in the upper left and a red + in the center with a line under it?"));
}

function testClear()
{
  jjs.Turtle.init(theCanvas);
  var t = new jjs.Turtle(theCanvas);
  t.ctx.fillStyle = '#aabbcc';
  t.penDown().forward(80);
  t.clear();
  assertTrue(confirm("Do you see a light yellow background with nothing in it?"));

  t.penDown().forward(80);
  t.clear('#DF0DB3');
  assertTrue(confirm("Do you see a purple background with nothing in it?"));
  assertEquals('#aabbcc', t.ctx.fillStyle);
}

function testDraw()
{
  var ctx = theCanvas.getContext('2d');
  jjs.Turtle.init(theCanvas);
  var t = new jjs.Turtle(theCanvas);

  // test forward and back by drawing horizontal lines
  t.penDown();
  t.moveTo(-40, 100);
  assertEquals(t.x, -40);
  assertEquals(t.y, 100);
  assertEquals(t.h, 0);
  t.forward(80);
  assertEquals(t.x, 40);
  assertEquals(t.y, 100);
  assertEquals(t.h, 0);
  t.moveTo(40, 90);
  t.forward(-80);
  t.moveTo(40, 80);
  t.back(80);
  t.moveTo(-40, 70);
  t.back(-80);
  assertEquals(t.x, 40);
  assertEquals(t.y, 70);
  assertEquals(t.h, 0);
  // test left and right
  t.home();
  t.forward(40);
  t.home().left(45).forward(40);
  t.home().right(-90).forward(40);
  t.home().right(40).right(5).forward(40);
  t.home().left(-45).left(-45).forward(40);
  t.home().heading(180).forward(40);
  t.moveTo(0,0).right(45+360).forward(40);
  t.moveTo(0,0).left(90).forward(40);

  assertTrue(confirm("Do you see 4 horizontal lines in the upper center"
  + " and 8 lines radiating from the center at 45 degree increments?"));

}

function testPen()
{
  var ctx = theCanvas.getContext('2d');
  jjs.Turtle.init(theCanvas);
  var t = new jjs.Turtle(theCanvas);

  // test width and style
  t.width(4).style('green').moveTo(-10, 40).penDown();
  t.forward(20).width(1).style('red').moveTo(-11.5, 43.5);
  t.forward(23).right(90).forward(7).right(90).forward(23).right(90).forward(7);

  // pen up and down
  t.moveTo(-100, 0).width(4).style('blue').heading(0);
  for (var i = 0; i < 10; i++)
  {
    t.penDown().forward(10);
    t.penUp().forward(10);
  }

  // width and style are independent from context width and style
  t.width(20).style('black');
  ctx.beginPath();
  ctx.moveTo(110,50);
  ctx.lineTo(110,-50);
  ctx.stroke();
  t.width(4).style('blue');
  ctx.lineWidth = 20;
  ctx.strokeStyle = 'black';
  t.moveTo(-110, 50).penDown();
  t.heading(-90);
  t.forward(100);

  assertTrue(confirm("Do you see a short green line with a thin red box around it"
    + " and dashed blue horizontal line and vertical lines that are blue not black?"));
}

function testStack()
{
  function drawStopSign()
  {
    t.left(90).forward(60);
    t.left(90).forward(5);
    for(var i = 0; i < 7; i++)
    {
      t.right(45).forward(10);
    }
    t.right(45).forward(5);
  }

  var ctx = theCanvas.getContext('2d');
  jjs.Turtle.init(theCanvas);
  var t = new jjs.Turtle(theCanvas);

  assertEquals(t.toString(), "{x: 0, y: 0, pen: up, penWidth: 2, penStyle: black, stack depth: 0}");
  t.push();
  t.moveTo(90,91).heading(30).penDown().width(7).style('green');
  assertEquals(t.toString(), "{x: 90, y: 91, pen: down, penWidth: 7, penStyle: green, stack depth: 1}");
  t.pop();
  assertEquals(t.toString(), "{x: 0, y: 0, pen: up, penWidth: 2, penStyle: black, stack depth: 0}");

  t.moveTo(-120, 0).heading(0).penDown();
  for (var i = 0; i < 5; i++)
  {
    t.forward(20).push();
    drawStopSign();
    t.pop();
    t.forward(20);
  }

  assertTrue(confirm("Do you see a horizontal line with 5 stop signs?"));
}

function testMoreStyles()
{
  var ctx = theCanvas.getContext('2d');
  jjs.Turtle.init(theCanvas);
  var t = new jjs.Turtle(theCanvas);
  
  t.clear('red');

  // line cap
  ctx.lineCap = 'butt';
  t.width(10).penDown();
  t.moveTo(-50, 60);
  t.forward(100);
  ctx.lineCap = 'round';
  t.moveTo(-50, 40);
  t.forward(100);
  ctx.lineCap = 'square';
  t.moveTo(-50, 20);
  t.forward(100);

  t.width(8).style('blue');
  ctx.globalAlpha = 0.2;
  for (var i = 0; i < 4; i++)
  {
    t.moveTo(-50, -20 - (i * 12));
    ctx.globalAlpha += 0.2;
    t.forward(100);
  }
  assertTrue(confirm("Do you see lines with 3 different line caps and blue lines with decreasing transparency?"));

  function zigzag()
  {
    t.left(45);
    t.forward(30);
    t.right(90);
    t.forward(30);
    t.left(90);
    t.forward(30);
    t.right(90);
    t.forward(30);
  }
}

function testTwoTurtles()
{
  var ctx = theCanvas.getContext('2d');
  jjs.Turtle.init(theCanvas);
  var t1 = new jjs.Turtle(theCanvas);
  var t2 = new jjs.Turtle(theCanvas);

  t1.moveTo(-100, 10);
  t2.moveTo(100, -10);
  t1.penDown().forward(88);
  t2.penDown().left(180).forward(88);
  t1.style('red').right(45).forward(60);
  t2.style('blue').right(45).forward(60);

  assertTrue(confirm("Do you see 2 lines one that bends up and one that bends down?"));
}


function runTests()
{
  \$.each([testInit,testClear,testDraw,testPen,testStack,testMoreStyles,testTwoTurtles]
         , function() {
         setUp();
         this();
         tearDown();
         });
}

\$(document).ready(function() {
  \$("#runtests").click(runTests);
});

</script>
</head>
<body>
	<h1>Turtle Testing</h1>
	<p>This page contains tests for the turtle.js module. <input type="button" value="run" id="runtests"></p>
	<div id="canvasHolder">
	</div>
</body>
</html>
